@@ -8,6 +8,10 @@ mod web_context;
 
 pub use web_context::WebContext;
 
+#[cfg(target_os = "android")]
+pub(crate) mod android;
+#[cfg(target_os = "android")]
+use android::*;
 #[cfg(any(
   target_os = "linux",
   target_os = "dragonfly",
@@ -33,6 +37,12 @@ pub(crate) mod webview2;
 #[cfg(target_os = "windows")]
 use self::webview2::*;
 use crate::Result;
+#[cfg(target_os = "android")]
+use jni::{
+  objects::{JClass, JObject},
+  sys::jobject,
+  JNIEnv,
+};
 #[cfg(target_os = "windows")]
 use webview2_com::Microsoft::Web::WebView2::Win32::ICoreWebView2Controller;
 #[cfg(target_os = "windows")]
@@ -110,6 +120,12 @@ pub struct WebViewAttributes {
   #[cfg(not(feature = "file-drop"))]
   file_drop_handler: Option<Box<dyn Fn(&Window, FileDropEvent) -> bool>>,
 
+  /// Set a navigation handler to decide if incoming url is allowed to navigate.
+  ///
+  /// The closure take a `String` parameter as url and return `bool` to determine the url. True is
+  /// allow to nivagate and false is not.
+  pub navigation_handler: Option<Box<dyn Fn(String) -> bool>>,
+
   /// Enables clipboard access for the page rendered on **Linux** and **Windows**.
   ///
   /// macOS doesn't provide such method and is always enabled by default. But you still need to add menu
@@ -124,7 +140,7 @@ pub struct WebViewAttributes {
   /// # Warning
   /// This will call private functions on **macOS**. It's still enabled if set in **debug** build on mac,
   /// but requires `devtool` feature flag to actually enable it in **release** build.
-  pub devtool: bool,
+  pub devtools: bool,
 }
 
 impl Default for WebViewAttributes {
@@ -139,8 +155,9 @@ impl Default for WebViewAttributes {
       custom_protocols: vec![],
       ipc_handler: None,
       file_drop_handler: None,
+      navigation_handler: None,
       clipboard: false,
-      devtool: false,
+      devtools: false,
     }
   }
 }
@@ -290,9 +307,18 @@ impl<'a> WebViewBuilder<'a> {
   ///
   /// # Warning
   /// This will call private functions on **macOS**. It's still enabled if set in **debug** build on mac,
-  /// but requires `devtool` feature flag to actually enable it in **release** build.
-  pub fn with_dev_tool(mut self, devtool: bool) -> Self {
-    self.webview.devtool = devtool;
+  /// but requires `devtools` feature flag to actually enable it in **release** build.
+  pub fn with_devtools(mut self, devtools: bool) -> Self {
+    self.webview.devtools = devtools;
+    self
+  }
+
+  /// Set a navigation handler to decide if incoming url is allowed to navigate.
+  ///
+  /// The closure takes a `String` parameter as url and return `bool` to determine the url. True is
+  /// allowed to nivagate and false is not.
+  pub fn with_navigation_handler(mut self, callback: impl Fn(String) -> bool + 'static) -> Self {
+    self.webview.navigation_handler = Some(Box::new(callback));
     self
   }
 
@@ -324,21 +350,26 @@ pub struct WebView {
 
 // Signal the Window to drop on Linux and Windows. On mac, we need to handle several unsafe code
 // blocks and raw pointer properly.
+#[cfg(any(
+  target_os = "linux",
+  target_os = "dragonfly",
+  target_os = "freebsd",
+  target_os = "netbsd",
+  target_os = "openbsd"
+))]
 impl Drop for WebView {
   fn drop(&mut self) {
-    #[cfg(any(
-      target_os = "linux",
-      target_os = "dragonfly",
-      target_os = "freebsd",
-      target_os = "netbsd",
-      target_os = "openbsd"
-    ))]
     unsafe {
       use crate::application::platform::unix::WindowExtUnix;
       use gtk::prelude::WidgetExtManual;
       self.window().gtk_window().destroy();
     }
-    #[cfg(target_os = "windows")]
+  }
+}
+
+#[cfg(target_os = "windows")]
+impl Drop for WebView {
+  fn drop(&mut self) {
     unsafe {
       DestroyWindow(HWND(self.window.hwnd() as _));
     }
@@ -399,8 +430,33 @@ impl WebView {
   }
 
   /// Open the web inspector which is usually called dev tool.
-  pub fn devtool(&self) {
-    self.webview.devtool();
+  ///
+  /// ## Platform-specific
+  ///
+  /// - **Android / iOS:** Not supported.
+  #[cfg(any(debug_assertions, feature = "devtools"))]
+  pub fn open_devtools(&self) {
+    self.webview.open_devtools();
+  }
+
+  /// Close the web inspector which is usually called dev tool.
+  ///
+  /// ## Platform-specific
+  ///
+  /// - **Windows / Android / iOS:** Not supported.
+  #[cfg(any(debug_assertions, feature = "devtools"))]
+  pub fn close_devtools(&self) {
+    self.webview.close_devtools();
+  }
+
+  /// Gets the devtool window's current vibility state.
+  ///
+  /// ## Platform-specific
+  ///
+  /// - **Windows / Android / iOS:** Not supported.
+  #[cfg(any(debug_assertions, feature = "devtools"))]
+  pub fn is_devtools_open(&self) -> bool {
+    self.webview.is_devtools_open()
   }
 
   pub fn inner_size(&self) -> PhysicalSize<u32> {
@@ -412,6 +468,16 @@ impl WebView {
     #[cfg(not(target_os = "macos"))]
     self.window.inner_size()
   }
+
+  #[cfg(target_os = "android")]
+  pub fn run(self, env: JNIEnv, jclass: JClass, jobject: JObject) -> jobject {
+    self.webview.run(env, jclass, jobject).unwrap()
+  }
+
+  #[cfg(target_os = "android")]
+  pub fn ipc_handler(window: &Window, arg: String) {
+    InnerWebView::ipc_handler(window, arg)
+  }
 }
 
 /// An event enumeration sent to [`FileDropHandler`].
@@ -435,13 +501,53 @@ pub fn webview_version() -> Result<String> {
 #[cfg(target_os = "windows")]
 pub trait WebviewExtWindows {
   /// Returns WebView2 Controller
-  fn controller(&self) -> Option<ICoreWebView2Controller>;
+  fn controller(&self) -> ICoreWebView2Controller;
 }
 
 #[cfg(target_os = "windows")]
 impl WebviewExtWindows for WebView {
-  fn controller(&self) -> Option<ICoreWebView2Controller> {
-    Some(self.webview.controller.clone())
+  fn controller(&self) -> ICoreWebView2Controller {
+    self.webview.controller.clone()
+  }
+}
+
+/// Additional methods on `WebView` that are specific to Linux.
+#[cfg(target_os = "linux")]
+pub trait WebviewExtUnix {
+  /// Returns Webkit2gtk Webview handle
+  fn webview(&self) -> Rc<webkit2gtk::WebView>;
+}
+
+#[cfg(target_os = "linux")]
+impl WebviewExtUnix for WebView {
+  fn webview(&self) -> Rc<webkit2gtk::WebView> {
+    self.webview.webview.clone()
+  }
+}
+
+/// Additional methods on `WebView` that are specific to macOS.
+#[cfg(target_os = "macOS")]
+pub trait WebviewExtMacOS {
+  /// Returns WKWebView handle
+  fn webview(&self) -> cocoa::base::id;
+  /// Returns WKWebView manager [(userContentController)](https://developer.apple.com/documentation/webkit/wkscriptmessagehandler/1396222-usercontentcontroller) handle
+  fn manager(&self) -> cocoa::base::id;
+  /// Returns NSWindow associated with the WKWebView webview
+  fn ns_window(&self) -> cocoa::base::id;
+}
+
+#[cfg(target_os = "macOS")]
+impl WebviewExtMacOS for WebView {
+  fn webview(&self) -> cocoa::base::id {
+    self.webview.webview.clone()
+  }
+
+  fn manager(&self) -> cocoa::base::id {
+    self.webview.manager.clone()
+  }
+
+  fn ns_window(&self) -> cocoa::base::id {
+    self.webview.ns_window.clone()
   }
 }
 
