@@ -2,6 +2,11 @@
 // SPDX-License-Identifier: Apache-2.0
 // SPDX-License-Identifier: MIT
 
+#[cfg(any(debug_assertions, feature = "devtools"))]
+use std::sync::{
+  atomic::{AtomicBool, Ordering},
+  Arc,
+};
 use std::{
   collections::hash_map::DefaultHasher,
   hash::{Hash, Hasher},
@@ -13,11 +18,12 @@ use gio::Cancellable;
 use glib::signal::Inhibit;
 use gtk::prelude::*;
 use webkit2gtk::{
-  traits::*, UserContentInjectedFrames, UserScript, UserScriptInjectionTime, WebView,
-  WebViewBuilder,
+  traits::*, NavigationPolicyDecision, PolicyDecisionType, UserContentInjectedFrames, UserScript,
+  UserScriptInjectionTime, WebView, WebViewBuilder,
 };
 use webkit2gtk_sys::{
   webkit_get_major_version, webkit_get_micro_version, webkit_get_minor_version,
+  webkit_policy_decision_ignore, webkit_policy_decision_use,
 };
 
 use web_context::WebContextExt;
@@ -33,7 +39,9 @@ mod file_drop;
 mod web_context;
 
 pub struct InnerWebView {
-  webview: Rc<WebView>,
+  pub(crate) webview: Rc<WebView>,
+  #[cfg(any(debug_assertions, feature = "devtools"))]
+  is_inspector_open: Arc<AtomicBool>,
 }
 
 impl InnerWebView {
@@ -198,6 +206,31 @@ impl InnerWebView {
       Inhibit(false)
     });
 
+    if let Some(nav_handler) = attributes.navigation_handler {
+      webview.connect_decide_policy(move |_webview, policy_decision, policy_type| {
+        if let PolicyDecisionType::NavigationAction = policy_type {
+          if let Some(policy) = policy_decision.dynamic_cast_ref::<NavigationPolicyDecision>() {
+            if let Some(nav_action) = policy.navigation_action() {
+              if let Some(uri_req) = nav_action.request() {
+                if let Some(uri) = uri_req.uri() {
+                  let allow = nav_handler(uri.to_string());
+                  let pointer = policy_decision.as_ptr();
+                  unsafe {
+                    if allow {
+                      webkit_policy_decision_use(pointer)
+                    } else {
+                      webkit_policy_decision_ignore(pointer)
+                    }
+                  }
+                }
+              }
+            }
+          }
+        }
+        true
+      });
+    }
+
     // Gtk application window can only contain one widget at a time.
     // In tao, we add a GtkBox to pack menu bar. So we check if
     // there's a box widget here.
@@ -225,7 +258,7 @@ impl InnerWebView {
       // Set user agent
       settings.set_user_agent(attributes.user_agent.as_deref());
 
-      if attributes.devtool {
+      if attributes.devtools {
         settings.set_enable_developer_extras(true);
       }
     }
@@ -244,7 +277,28 @@ impl InnerWebView {
       window.show_all();
     }
 
-    let w = Self { webview };
+    #[cfg(any(debug_assertions, feature = "devtools"))]
+    let is_inspector_open = {
+      let is_inspector_open = Arc::new(AtomicBool::default());
+      if let Some(inspector) = WebViewExt::inspector(&*webview) {
+        let is_inspector_open_ = is_inspector_open.clone();
+        inspector.connect_bring_to_front(move |_| {
+          is_inspector_open_.store(true, Ordering::Relaxed);
+          false
+        });
+        let is_inspector_open_ = is_inspector_open.clone();
+        inspector.connect_closed(move |_| {
+          is_inspector_open_.store(false, Ordering::Relaxed);
+        });
+      }
+      is_inspector_open
+    };
+
+    let w = Self {
+      webview,
+      #[cfg(any(debug_assertions, feature = "devtools"))]
+      is_inspector_open,
+    };
 
     // Initialize message handler
     let mut init = String::with_capacity(115 + 20 + 22);
@@ -313,11 +367,28 @@ impl InnerWebView {
   }
 
   /// Open the web inspector which is usually called dev tool.
-  pub fn devtool(&self) {
+  #[cfg(any(debug_assertions, feature = "devtools"))]
+  pub fn open_devtools(&self) {
     if let Some(inspector) = WebViewExt::inspector(&*self.webview) {
       inspector.show();
+      // `bring-to-front` is not received in this case
+      self.is_inspector_open.store(true, Ordering::Relaxed);
     }
   }
+
+  /// Close the web inspector which is usually called dev tool.
+  #[cfg(any(debug_assertions, feature = "devtools"))]
+  pub fn close_devtools(&self) {
+    if let Some(inspector) = WebViewExt::inspector(&*self.webview) {
+      inspector.close();
+    }
+  }
+
+  /// Gets the devtool window's current vibility state.
+  #[cfg(any(debug_assertions, feature = "devtools"))]
+  pub fn is_devtools_open(&self) -> bool {
+    self.is_inspector_open.load(Ordering::Relaxed)
+  }
 }
 
 pub fn platform_webview_version() -> Result<String> {
