@@ -216,8 +216,8 @@ impl InnerWebView {
       settings
         .SetAreDevToolsEnabled(false)
         .map_err(webview2_com::Error::WindowsError)?;
-      if attributes.devtool {
-        settings.SetAreDevToolsEnabled(true);
+      if attributes.devtools {
+        let _ = settings.SetAreDevToolsEnabled(true);
       }
 
       let mut rect = RECT::default();
@@ -301,6 +301,29 @@ window.addEventListener('mousemove', (e) => window.chrome.webview.postMessage('_
     }
     .map_err(webview2_com::Error::WindowsError)?;
 
+    if let Some(nav_callback) = attributes.navigation_handler {
+      unsafe {
+        webview
+          .NavigationStarting(
+            NavigationStartingEventHandler::create(Box::new(move |_, args| {
+              if let Some(args) = args {
+                let mut uri = PWSTR::default();
+                args.Uri(&mut uri)?;
+                let uri = take_pwstr(uri);
+
+                let allow = nav_callback(uri);
+
+                args.SetCancel(!allow)?;
+              }
+
+              Ok(())
+            })),
+            &mut token,
+          )
+          .map_err(webview2_com::Error::WindowsError)?;
+      }
+    }
+
     let mut custom_protocol_names = HashSet::new();
     if !attributes.custom_protocols.is_empty() {
       for (name, _) in &attributes.custom_protocols {
@@ -578,10 +601,19 @@ window.addEventListener('mousemove', (e) => window.chrome.webview.postMessage('_
   }
 
   /// Open the web inspector which is usually called dev tool.
-  pub fn devtool(&self) {
-    let _ = unsafe {
-      self.webview.OpenDevToolsWindow();
-    };
+  #[cfg(any(debug_assertions, feature = "devtools"))]
+  pub fn open_devtools(&self) {
+    let _ = unsafe { self.webview.OpenDevToolsWindow() };
+  }
+
+  /// Close the web inspector which is usually called dev tool.
+  #[cfg(any(debug_assertions, feature = "devtools"))]
+  pub fn close_devtools(&self) {}
+
+  /// Gets the devtool window's current vibility state.
+  #[cfg(any(debug_assertions, feature = "devtools"))]
+  pub fn is_devtools_open(&self) -> bool {
+    false
   }
 }
 
